﻿using CommandLine;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;

namespace Bosch.RPS.API.Examples
{
    internal class Program
    {
        private static HttpClient _httpClient;

        private static async Task Main(string[] args)

        {
            if (args == null)
                throw new ArgumentNullException(nameof(args));

            if (args.Length == 0)
            {
                args = new string[] { "-s", "https://" + Environment.MachineName + ":9000", "-u", "admin", "-p", "1111" };
            }

            var _option = Parser.Default.ParseArguments<Options>(args);

            await _option.MapResult(async o => { await RunExamples(o); }, errors => Task.FromResult(0));
        }

        private static async Task RunExamples(Options o)
        {
            if (o is null)
                throw new ArgumentNullException(nameof(o));

            _httpClient = new HttpClient();

            var apiVersion = await GetApiVersion(o.ServerUrl);
            Console.WriteLine("\nApi Version: \n");
            Console.WriteLine("\t" + apiVersion);

            var token = await GetToken(o.ServerUrl, o.Username, o.Password);
            Console.WriteLine("\\nToken: \n");
            Console.WriteLine("\t" + token);
            var accessToken = JObject.Parse(token);
            var panels = await GetPanels(o.ServerUrl, accessToken["access_token"].ToString());
            var panelList = JObject.Parse(panels);
            Console.WriteLine("\nPanels:\n");
            foreach (var p in panelList["returnObject"])
            {
                Console.WriteLine("\t" + p["name"] + "\t\t" + p["guid"]);
            }

            return;
        }

        private static async Task<string> GetApiVersion(string url)
        {
            try
            {
                HttpResponseMessage response = await _httpClient.GetAsync(url + "/rpsapi/apiVersion");
                response.EnsureSuccessStatusCode();
                var result = await response.Content.ReadAsStringAsync();
                return result;
            }
            catch (HttpRequestException e)
            {
                Console.WriteLine("Exception: {0}", e.Message);
                throw;
            }
        }

        private static async Task<string> GetToken(string url, string username, string password)
        {
            try
            {
                var dict = new Dictionary<string, string>();
                dict.Add("username", username);
                dict.Add("password", password);
                dict.Add("grant_type", "password");

                var request = new HttpRequestMessage(HttpMethod.Post, url + "/rpsapi/token")
                {
                    Content = new FormUrlEncodedContent(dict)
                };

                HttpResponseMessage response = await _httpClient.SendAsync(request);
                response.EnsureSuccessStatusCode();
                var result = await response.Content.ReadAsStringAsync();
                return result;
            }
            catch (HttpRequestException e)
            {
                Console.WriteLine("Exception: {0}", e.Message);
                throw;
            }
        }

        private static async Task<string> GetPanels(string url, string token)
        {
            try
            {
                var request = new HttpRequestMessage
                {
                    Method = HttpMethod.Get,
                    RequestUri = new Uri(url + "/rpsapi/v2/panels"),
                    Headers = {
                        { HttpRequestHeader.Authorization.ToString(), "Bearer " + token },
                        { HttpRequestHeader.Accept.ToString(), "application/json" },
                    },
                };

                HttpResponseMessage response = await _httpClient.SendAsync(request);
                response.EnsureSuccessStatusCode();
                var result = await response.Content.ReadAsStringAsync();
                return result;
            }
            catch (HttpRequestException e)
            {
                Console.WriteLine("Exception: {0}", e.Message);
                throw;
            }
        }
    }
}